Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms
Imports Telerik.WinControls.UI
Imports NorthwindLibrary
Imports Telerik.WinControls.Data
Imports System.Reflection
Imports Telerik.OpenAccess
Imports System.ComponentModel

Namespace WinFormsApplication
	Public Class FormController
		Private view_Renamed As CategoryForm
		Private context As NorthwindOAEntityDiagrams

		Public Sub New()
			Me.SubscribeToEvents()

			Me.context = ContextFactory.ObtainContext()
			Me.View.CategorySource = Me.context.Categories.ToList()
			Me.View.ConfigureGrid()
		End Sub

		Public ReadOnly Property View() As CategoryForm
			Get
				If Me.view_Renamed Is Nothing Then
					Me.view_Renamed = New CategoryForm()
				End If
				Return Me.view_Renamed
			End Get
		End Property

		Private Sub SubscribeToEvents()
			AddHandler View.DataSourceCollectionChanged, AddressOf View_DataSourceCollectionChanged
			AddHandler View.RowsChanging, AddressOf View_RowsChanging

			AddHandler View.SaveChangesButtonClicked, AddressOf View_SaveChangesButtonClicked
			AddHandler View.CategoryFormClosed, AddressOf View_CategoryFormClosed
		End Sub

		Private Sub View_DataSourceCollectionChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ListChangedEventArgs)
			If e.ListChangedType = ListChangedType.ItemAdded Then
				Dim source As BindingSource = TryCast(sender, BindingSource)
				If e.NewIndex >= 0 AndAlso e.NewIndex < source.List.Count Then
					Dim newCategory As Category = TryCast(source.List(e.NewIndex), Category)
					Me.context.Add(newCategory)
				End If
			End If
		End Sub

		Private Sub View_RowsChanging(ByVal sender As Object, ByVal e As GridViewCollectionChangingEventArgs)
			If e.Action = NotifyCollectionChangedAction.Remove Then
				Dim dialogResult As DialogResult = MessageBox.Show("Do you want to delete this category?", "Warning", MessageBoxButtons.OKCancel, MessageBoxIcon.Question)
				If dialogResult <> System.Windows.Forms.DialogResult.OK Then
					e.Cancel = True
				Else
					Dim category As Category = Me.View.SelectedCategory
					If Not category Is Nothing Then
						context.Delete(category)
					End If
				End If
			End If
		End Sub

		Private Sub View_SaveChangesButtonClicked(ByVal sender As Object, ByVal e As EventArgs)
			Try
				Me.View.EndEdit()
				Me.context.SaveChanges()
				Dim dialogResult As DialogResult = MessageBox.Show("Changes were successfully saved.", "Result", MessageBoxButtons.OKCancel, MessageBoxIcon.Information)
			Catch ex As Exception
				Dim message As String = ex.Message
				Dim dialogResult As DialogResult = MessageBox.Show(String.Format("An error occurred during the save operation:" & Constants.vbLf & "{0}", message), "Result", MessageBoxButtons.OKCancel, MessageBoxIcon.Error)
			End Try
		End Sub

		Private Sub View_CategoryFormClosed(ByVal sender As Object, ByVal e As FormClosedEventArgs)
			If Not Me.context Is Nothing Then
				Me.context.Dispose()
			End If
		End Sub
	End Class
End Namespace
